import PropTypes from 'prop-types';
import { mergeDeepWith, flip, or, compose, o, cond, isNil, apply, __, map, T, dissoc, mergeLeft } from 'ramda';
import React, { createContext, useContext, useMemo } from 'react';
import { Provider, connect, useDispatch, useSelector } from 'react-redux';
import { DEFAULT_FEATURE, getStateByFeatureAndNamespace, defaultNamespace } from '@redux-tools/namespaces';
import { alwaysNull, alwaysEmptyObject, isFunction, isObject } from 'ramda-extension';
import { withProps, mapProps, getDisplayName } from '@redux-tools/utils-react';
import hoistNonReactStatics from 'hoist-non-react-statics';

var NamespaceContext = createContext({
  namespaces: {},
  useNamespace: alwaysNull,
  isUseNamespaceProvided: false
});

var mergeContextValues = mergeDeepWith(flip(or));

var NamespaceProvider = function NamespaceProvider(_ref) {
  var children = _ref.children,
      feature = _ref.feature,
      namespace = _ref.namespace,
      store = _ref.store,
      useNamespace = _ref.useNamespace;
  var context = useContext(NamespaceContext);
  var nextContext = useMemo(function () {
    var _namespaces;

    return mergeContextValues(context, {
      // NOTE: Defaulting here is safer than using `Provider.defaultProps`,
      // because passing `null` would not result in a fallback to `DEFAULT_FEATURE`.
      namespaces: (_namespaces = {}, _namespaces[feature != null ? feature : DEFAULT_FEATURE] = namespace, _namespaces),
      useNamespace: useNamespace,
      isUseNamespaceProvided: Boolean(useNamespace)
    });
  }, [context, feature, namespace, useNamespace]);
  var providerElement = React.createElement(NamespaceContext.Provider, {
    value: nextContext
  }, children);
  return store ? React.createElement(Provider, {
    store: store
  }, providerElement) : providerElement;
};

NamespaceProvider.propTypes = {
  children: PropTypes.node.isRequired,
  feature: PropTypes.string,
  namespace: PropTypes.string,
  store: PropTypes.object,
  useNamespace: PropTypes.func
};

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var useNamespace = function useNamespace(feature) {
  var _ref, _namespaces$feature;

  var _useContext = useContext(NamespaceContext),
      _useContext$namespace = _useContext.namespaces,
      namespaces = _useContext$namespace === void 0 ? {} : _useContext$namespace,
      _useContext$useNamesp = _useContext.useNamespace,
      useNamespace = _useContext$useNamesp === void 0 ? alwaysNull : _useContext$useNamesp;

  var namespace = useNamespace(feature != null ? feature : DEFAULT_FEATURE, namespaces);
  return (_ref = (_namespaces$feature = namespaces[feature]) != null ? _namespaces$feature : namespace) != null ? _ref : null;
};

var NAMESPACED_CONNECT_PROPS = 'NAMESPACED_CONNECT_PROPS';
var wrapMapStateToProps = function wrapMapStateToProps(mapStateToProps) {
  return function (state, ownProps) {
    return mapStateToProps ? mapStateToProps(getStateByFeatureAndNamespace(ownProps[NAMESPACED_CONNECT_PROPS].feature, ownProps[NAMESPACED_CONNECT_PROPS].namespace, state), ownProps, state) : {};
  };
};

var wrapActionCreator = function wrapActionCreator(wrappedDispatch) {
  return function (actionCreator) {
    return compose(wrappedDispatch, actionCreator);
  };
};

var throwTypeError = function throwTypeError() {
  throw new TypeError('mapDispatchToProps is not an object or a function');
};

var wrapMapDispatchToProps = function wrapMapDispatchToProps(mapDispatchToProps) {
  return function (dispatch, ownProps) {
    var wrappedDispatch = o(dispatch, defaultNamespace(ownProps[NAMESPACED_CONNECT_PROPS].namespace));
    return cond([[isNil, alwaysEmptyObject], [isFunction, apply(__, [wrappedDispatch, ownProps])], [isObject, map(wrapActionCreator(wrappedDispatch))], [T, throwTypeError]])(mapDispatchToProps);
  };
};

var rawNamespacedConnect = function rawNamespacedConnect(mapStateToProps, mapDispatchToProps) {
  for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  return connect.apply(void 0, [wrapMapStateToProps(mapStateToProps), wrapMapDispatchToProps(mapDispatchToProps)].concat(args));
};

var namespacedConnect = function namespacedConnect(mapStateToProps, mapDispatchToProps, mergeProps, _ref) {
  if (_ref === void 0) {
    _ref = {};
  }

  var _ref2 = _ref,
      optionFeature = _ref2.feature,
      optionNamespace = _ref2.namespace,
      options = _objectWithoutPropertiesLoose(_ref2, ["feature", "namespace"]);

  return compose(withProps(function (_ref3) {
    var _ref4, _ref5, _ref6;

    var propFeature = _ref3.feature,
        propNamespace = _ref3.namespace;
    var feature = (_ref4 = optionFeature != null ? optionFeature : propFeature) != null ? _ref4 : DEFAULT_FEATURE;
    var contextNamespace = useNamespace(feature);
    return _ref6 = {}, _ref6[NAMESPACED_CONNECT_PROPS] = {
      feature: feature,
      namespace: (_ref5 = optionNamespace != null ? optionNamespace : propNamespace) != null ? _ref5 : contextNamespace
    }, _ref6;
  }), rawNamespacedConnect(mapStateToProps, mapDispatchToProps, mergeProps, options), mapProps(dissoc(NAMESPACED_CONNECT_PROPS)));
};

var useNamespacedDispatch = function useNamespacedDispatch(_temp) {
  var _ref2;

  var _ref = _temp === void 0 ? {} : _temp,
      optionFeature = _ref.feature,
      optionNamespace = _ref.namespace;

  var dispatch = useDispatch();
  var feature = optionFeature != null ? optionFeature : DEFAULT_FEATURE;
  var contextNamespace = useNamespace(feature);
  var namespace = (_ref2 = optionNamespace != null ? optionNamespace : contextNamespace) != null ? _ref2 : null;
  return useMemo(function () {
    return o(dispatch, defaultNamespace(namespace));
  }, [dispatch, namespace]);
};

var useNamespacedSelector = function useNamespacedSelector(selector, equalityFn, _temp) {
  var _ref2;

  var _ref = _temp === void 0 ? {} : _temp,
      optionFeature = _ref.feature,
      optionNamespace = _ref.namespace;

  var feature = optionFeature != null ? optionFeature : DEFAULT_FEATURE;
  var contextNamespace = useNamespace(feature);
  var namespace = (_ref2 = optionNamespace != null ? optionNamespace : contextNamespace) != null ? _ref2 : null;
  var namespacedSelector = useMemo(function () {
    return o(selector, getStateByFeatureAndNamespace(feature, namespace));
  }, [selector, feature, namespace]);
  return useSelector(namespacedSelector, equalityFn);
};

var withNamespaceProvider = function withNamespaceProvider(options) {
  return function (NextComponent) {
    var WithNamespaceProvider = function WithNamespaceProvider(props) {
      return (// NOTE: `NamespaceProvider` will ignore any unknown props.
        React.createElement(NamespaceProvider, mergeLeft(options, props), React.createElement(NextComponent, props))
      );
    };

    hoistNonReactStatics(WithNamespaceProvider, NextComponent);
    WithNamespaceProvider.displayName = "WithNamespaceProvider(" + getDisplayName(NextComponent) + ")";
    return WithNamespaceProvider;
  };
};

export { NamespaceContext, NamespaceProvider, namespacedConnect, useNamespace, useNamespacedDispatch, useNamespacedSelector, withNamespaceProvider };
